
/**
 * CSCI 201 Lab 5.
 * @author CSCI 201 Instructors.
 */
import java.awt.Frame;
import java.awt.Label;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.Font;
import java.awt.Color;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.Scanner;
import java.io.FileWriter;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;

public class TextEditor implements KeyListener {

  // constants
  private static final int WIDTH = 500;
  private static final int HEIGHT = 500;
  private static final int FONT_SIZE = 16;
  private static final int MAX_LINES = 30;

  private Frame frame;
  private Label[] labels;
  private ArrayList<Line> lines;
  private int currentLineIndex;

  /**
   * Initializes the text editor lines.
   */
  public TextEditor() {
    lines = new ArrayList<>();
    lines.add(new Line());
    currentLineIndex = 0;
  }

  /**
   * Sets up the frame and labels (in which lines will be written)
   * for the text editor.
   */
  public void run() {
    frame = new Frame("MiddDocs");
    frame.setLayout(new GridLayout(MAX_LINES, 1));
    frame.setSize(WIDTH, HEIGHT);
    frame.setBackground(Color.WHITE);

    labels = new Label[MAX_LINES];
    for (int i = 0; i < MAX_LINES; i++) {
      labels[i] = new Label("", Label.LEFT);
      labels[i].setFont(new Font("Courier", Font.PLAIN, FONT_SIZE));
      frame.add(labels[i]);
    }

    frame.setVisible(true);
    frame.validate();
    frame.addKeyListener(this);
    refresh();
  }

  /**
   * Listens for events in which the user types.
   * This isn't used but needs to be implemented
   * for the KeyListener interface.
   */
  @Override
  public void keyTyped(KeyEvent e) {
  }

  /**
   * Listens for a key press.
   */
  @Override
  public void keyPressed(KeyEvent e) {
    if (e.getKeyCode() == KeyEvent.VK_ESCAPE) {
      Scanner scanner = new Scanner(System.in);
      System.out.print("Save file? [y]n ");
      String input = scanner.next();
      if (input.toLowerCase().equals("y")) {
        System.out.print("Enter filename: ");
        String filename = scanner.next();
        if (filename.length() == 0)
          filename = "Untitled.txt";
        save(filename);
      }
      scanner.close();
      System.exit(0);
    } else if (e.getKeyCode() == KeyEvent.VK_SHIFT) {
      // check for other key modifiers?
    } else if (e.getKeyCode() == KeyEvent.VK_LEFT) {
      lines.get(currentLineIndex).moveCursorLeft();
    } else if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
      lines.get(currentLineIndex).moveCursorRight();
    } else if (e.getKeyCode() == KeyEvent.VK_DELETE || e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
      Line line = lines.get(currentLineIndex);
      if (line.head == null) {
        lines.remove(currentLineIndex);
        currentLineIndex--;
        if (currentLineIndex < 0) {
          lines.add(new Line());
          currentLineIndex = 0;
        }
      } else {
        lines.get(currentLineIndex).delete();
      }
    } else if (e.getKeyCode() == KeyEvent.VK_ENTER) {
      if (currentLineIndex + 1 < MAX_LINES) {
        lines.add(++currentLineIndex, new Line());
      } else {
        System.out.println("Maximum number of lines reached.");
      }
    } else if (e.getKeyCode() == KeyEvent.VK_UP) {
      currentLineIndex--;
      if (currentLineIndex < 0)
        currentLineIndex = 0;
    } else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
      currentLineIndex++;
      if (currentLineIndex >= lines.size())
        currentLineIndex = lines.size() - 1;
    } else {
      lines.get(currentLineIndex).insert(e.getKeyChar());
    }
    // Refresh the text in the window.
    refresh();
  }

  /**
   * Listens for a key released.
   */
  @Override
  public void keyReleased(KeyEvent e) {
  }

  /**
   * Refreshes the text that is currently saved.
   */
  private void refresh() {
    for (int i = 0; i < lines.size(); i++) {
      labels[i].setText(lines.get(i).get(i == currentLineIndex));
    }
    for (int i = lines.size(); i < labels.length; i++) {
      labels[i].setText("");
    }
    frame.revalidate();
  }

  /**
   * Saves the current lines to a file.
   * 
   * @param filename - name of the file to write.
   */
  private void save(String filename) {
    try {
      FileWriter writer = new FileWriter(filename);
      for (Line line : lines) {
        // do not add the cursor (|) when saving a line
        writer.write(line.get(false) + "\n");
      }
      writer.close();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  /**
   * Creates a line for each line in the file.
   * 
   * @param filename - file to load.
   */
  public void read(String filename) {
    try {
      File file = new File(filename);
      Scanner scanner = new Scanner(file);
      lines.clear();
      while (scanner.hasNextLine()) {
        String data = scanner.nextLine();
        Line line = new Line(data);
        lines.add(line);
      }
      scanner.close();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    }
  }

  /**
   * Runs the text editor.
   * 
   * @param args - args[0] may contain the filename to load.
   */
  public static void main(String[] args) {
    TextEditor editor = new TextEditor();
    if (args.length == 1) {
      editor.read(args[0]); // load the file
    }
    editor.run();
  }
}
